@@ -19,6 +19,7 @@ from tqdm import tqdm
 
 from definitions import DATA_DIR
 from sckg.buildtrie import Buildtrim
+import en_core_web_sm
 from sckg.constant_util import ConstantUtil
 
 
@@ -33,6 +34,7 @@ class SoftwareKG:
             for line in f:
                 data_line = line.split(':')
                 self.re_weight[data_line[0]] = data_line[1]
+        self.nlp = en_core_web_sm.load()
 
     def is_exist_concept(self, word):
         if not self.graph.find_nodes_by_property("concept_name", word):
@@ -242,8 +244,13 @@ class SoftwareKG:
         # 给一段文本，返回在KG里面出现的概念
         concepts = []
         sentence = sentence.lower()
-        sentence = sentence.translate(str.maketrans('', '', string.punctuation))
-        words = sentence.strip("\n").lower().split()
+        # sentence = sentence.translate(str.maketrans('', '', string.punctuation))
+        # words = sentence.strip("\n").lower().split()
+        sentence = sentence.translate(str.maketrans('', '', string.punctuation)).strip("\n")
+        doc = self.nlp(sentence)
+        words = []
+        for token in doc:
+            words.append(token.lemma_)
         for i, word in enumerate(words):
             concept = word
             if i == len(words) - 1:
@@ -270,8 +277,13 @@ class SoftwareKG:
         # 给一段文本，返回在KG里面出现的有效最长概念
         concepts = []
         sentence = sentence.lower()
-        sentence = sentence.translate(str.maketrans('', '', string.punctuation))
-        words = sentence.strip("\n").lower().split()
+        sentence = sentence.translate(str.maketrans('', '', string.punctuation)).strip("\n")
+        doc = self.nlp(sentence)
+        words = []
+        for token in doc:
+            words.append(token.lemma_)
+        # sentence = sentence.translate(str.maketrans('', '', string.punctuation))
+        # words = sentence.strip("\n").lower().split()
         index = 0
         while index < len(words):
             concept = words[index]
@@ -296,8 +308,11 @@ class SoftwareKG:
         # 给一段文本，返回在KG里面出现的有效最长概念
         concepts = []
         sentence = sentence.lower()
-        sentence = sentence.translate(str.maketrans('', '', string.punctuation))
-        words = sentence.strip("\n").lower().split()
+        sentence = sentence.translate(str.maketrans('', '', string.punctuation)).strip("\n")
+        doc = self.nlp(sentence)
+        words = []
+        for token in doc:
+            words.append(token.lemma_)
         index = 0
         while index < len(words):
             concept = words[index]
@@ -318,8 +333,11 @@ class SoftwareKG:
         # 给一段文本，返回在KG里面出现的最长概念
         concepts = []
         sentence = sentence.lower()
-        sentence = sentence.translate(str.maketrans('', '', string.punctuation))
-        words = sentence.strip("\n").lower().split()
+        sentence = sentence.translate(str.maketrans('', '', string.punctuation)).strip("\n")
+        doc = self.nlp(sentence)
+        words = []
+        for token in doc:
+            words.append(token.lemma_)
         index = 0
         while index < len(words):
             concept = words[index]
