@@ -24,17 +24,143 @@ import en_core_web_sm
 from nltk.corpus import stopwords
 
 from definitions import DATA_DIR
+from sckg.buildtrie import Buildtrim
 from sckg.constant_util import ConstantUtil
 
 
 class SoftwareKGSearcher:
+    def __init__(self):
+        self.nlp = en_core_web_sm.load()
+        self.stopwords = set(stopwords.words('english'))
+
+    def __is_stop_word(self, word):
+        return True if word in self.stopwords else False
+
+    def is_exist_concept(self, concept):
+        pass
+
+    def is_concept(self, word):
+        pass
+
+    def find_all_concept_from_sentence(self, sentence):
+        """
+        给一段文本，返回在KG里面出现的所有概念
+        """
+        concepts = []
+        sentence = sentence.lower().translate(str.maketrans('', '', string.punctuation)).strip("\n")
+        doc = self.nlp(sentence)
+        words = []
+        for token in doc:
+            words.append(token.lemma_)
+        # words = sentence.strip("\n").lower().split()
+        for i, word in enumerate(words):
+            concept = word
+            if i == len(words) - 1:
+                break
+            for j in range(i + 1, i + 5):
+                if j < len(words):
+                    concept += " " + words[j]
+                if self.is_concept(concept):
+                    concepts.append(concept)
+        for word in words:
+            if not self.is_concept(word):
+                continue
+            concepts.append(word)
+        return set(concepts)
+
+    def find_longest_valid_concept_from_sentence(self, sentence):
+        """
+        给一段文本，返回在KG里面出现的有效最长概念
+        """
+        concepts = []
+        sentence = sentence.lower().translate(str.maketrans('', '', string.punctuation)).strip("\n")
+        doc = self.nlp(sentence)
+        words = []
+        for token in doc:
+            words.append(token.lemma_)
+        # words = sentence.strip("\n").lower().split()
+        index = 0
+        while index < len(words):
+            concept = words[index]
+            longest_concept = concept
+            for j in range(index + 1, index + 6):
+                if j < len(words):
+                    concept += " " + words[j]
+                    if self.is_exist_concept(concept):
+                        longest_concept = concept
+                        index = j
+                    else:
+                        break
+            if self.is_concept(longest_concept):
+                concepts.append(longest_concept)
+            index = index + 1
+        return concepts
+
+    def find_longest_valid_concept_non_stop_words_from_sentence(self, sentence):
+        """
+        给一段文本，返回在KG里面出现的去除停用词的有效最长概念
+        """
+        concepts = []
+        sentence = sentence.lower().translate(str.maketrans('', '', string.punctuation)).strip("\n")
+        doc = self.nlp(sentence)
+        words = []
+        for token in doc:
+            words.append(token.lemma_)
+        index = 0
+        while index < len(words):
+            concept = words[index]
+            longest_concept = concept
+            for j in range(index + 1, index + 6):
+                if j < len(words):
+                    concept += ' ' + words[j]
+                    if self.is_exist_concept(concept):
+                        longest_concept = concept
+                        index = j
+                    else:
+                        break
+            if self.is_concept(longest_concept) and longest_concept not in self.stopwords:
+                concepts.append(longest_concept)
+            index = index + 1
+        return concepts
+
+    def find_longest_concept_from_sentence(self, sentence):
+        """
+        给一段文本，返回在KG里面出现的最长概念
+        """
+        concepts = []
+        sentence = sentence.lower().translate(str.maketrans('', '', string.punctuation)).strip("\n")
+        doc = self.nlp(sentence)
+        words = []
+        for token in doc:
+            words.append(token.lemma_)
+        # words = sentence.strip("\n").lower().split()
+        index = 0
+        while index < len(words):
+            concept = words[index]
+            longest_concept = concept
+            for j in range(index + 1, index + 6):
+                if j < len(words):
+                    concept += " " + words[j]
+                    if self.is_exist_concept(concept):
+                        longest_concept = concept
+                        index = j
+                    else:
+                        break
+            if self.is_concept(longest_concept):
+                concepts.append((longest_concept, 'valid'))
+            else:
+                concepts.append((longest_concept, 'invalid'))
+            index = index + 1
+        return concepts
+
+
+class SoftwareKGSearcherRemote(SoftwareKGSearcher):
     def __init__(self, uri, user, password):
+        super(SoftwareKGSearcherRemote, self).__init__()
         self.constant = ConstantUtil()
         self.graph = Graph(uri, auth=(user, password))
         self.relation_weight = self.__construct_relation_weight()
-        self.concepts = self.__construct_concepts()
         self.nlp = en_core_web_sm.load()
-        self.stopwords = set(stopwords.words('english'))
 
     def dumps_entity2csv(self, label, properties: list, limit_size=None):
         # TODO: dump pagerank_score next
@@ -57,21 +183,6 @@ class SoftwareKGSearcher:
         cypher += 'RETURN file, nodes, relationships, properties, data;'
         self.graph.run(cypher)
 
-    @staticmethod
-    def __construct_concepts():
-        if (Path(DATA_DIR) / 'concept.bin').exists():
-            with open(Path(DATA_DIR) / 'concept.bin', 'rb') as rf:
-                return pickle.load(rf)
-        concepts = {}
-        with open(Path(DATA_DIR) / 'concept.csv', 'r', encoding='utf-8') as rf:
-            reader = csv.reader(rf)
-            next(reader)
-            for row in reader:
-                concepts[row[2]] = (row[1], row[0])
-        with open(Path(DATA_DIR) / 'concept.bin', 'wb') as wf:
-            pickle.dump(concepts, wf)
-        return concepts
-
     @staticmethod
     def __construct_relation_weight():
         relation_weight = {}
@@ -111,18 +222,10 @@ class SoftwareKGSearcher:
         cursor: Cursor = self.graph.run(query)
         return cursor.evaluate()
 
-    def get_id_by_concept_local(self, concept):
-        if not self.is_exist_concept_local(concept):
-            return None
-        return self.concepts[concept][0]
-
     def is_exist_concept(self, concept):
         query = f'MATCH (n:entity{{concept_name:{repr(concept)}}}) RETURN COUNT(*)'
         return True if self.graph.run(query).evaluate() > 0 else False
 
-    def is_exist_concept_local(self, concept):
-        return self.concepts.__contains__(concept)
-
     def is_exist_concept_by_id(self, concept_id):
         query = f'MATCH (n:entity{{id:{concept_id}}}) RETURN COUNT(*)'
         return True if self.graph.run(query).evaluate() > 0 else False
@@ -136,50 +239,27 @@ class SoftwareKGSearcher:
         return self.graph.run(query).evaluate()
 
     def get_concept_labels(self, concept):
-        query = f'MATCH (n:concept{{concept_name:{repr(concept)}}}) RETURN labels(n)'
+        query = f'MATCH (n:entity:concept{{concept_name:{repr(concept)}}}) RETURN labels(n)'
         return self.graph.run(query).evaluate()
 
-    def get_concept_labels_local(self, concept):
-        if not self.is_exist_concept_local(concept):
-            return None
-        return self.concepts[concept][1]
-
     def is_action(self, word):
         labels = self.get_concept_labels(word)
         if not labels:
             return False
         return True if 'action' in self.get_concept_labels(word) else False
 
-    def is_action_local(self, word):
-        labels = self.get_concept_labels_local(word)
-        if not labels:
-            return False
-        return True if 'action' in labels else False
-
     def is_characteristic(self, word):
         labels = self.get_concept_labels(word)
         if not labels:
             return False
         return True if 'characteristic' in labels else False
 
-    def is_characteristic_local(self, word):
-        labels = self.get_concept_labels_local(word)
-        if not labels:
-            return False
-        return True if 'characteristic' in labels else False
-
     def is_concept(self, word):
         labels = self.get_concept_labels(word)
         if not labels:
             return False
         return False if 'action' in labels or 'characteristic' in labels else True
 
-    def is_concept_local(self, word):
-        labels = self.get_concept_labels_local(word)
-        if not labels:
-            return False
-        return False if 'action' in labels or 'characteristic' in labels else True
-
     def get_out_relation_by_concept(self, concept):
         query = f'MATCH (n:entity{{concept_name:{repr(concept)}}})-[r]->(x) RETURN n.concept_name, type(r), x.concept_name'
         cursor: Cursor = self.graph.run(query)
@@ -191,12 +271,6 @@ class SoftwareKGSearcher:
         """
         query = f'MATCH (n:entity{{concept_name:{repr(concept)}}})-[r]->(x) RETURN n.concept_name, type(r), x'
         cursor: Cursor = self.graph.run(query)
-        # return [(record[0], record[1],
-        #          {
-        #              'id': record[2]['id'],
-        #              'labels': list(record[2].labels),
-        #              'properties': dict(record[2])
-        #          }) for record in cursor]
         result = []
         for record in cursor:
             node: Node = record[2]
@@ -222,11 +296,6 @@ class SoftwareKGSearcher:
         """
         query = f'MATCH (n:entity{{concept_name:{repr(concept)}}})<-[r]-(x) RETURN x, type(r), n.concept_name'
         cursor: Cursor = self.graph.run(query)
-        # return [({
-        #              'id': record[0]['id'],
-        #              'labels': list(record[0].labels),
-        #              'properties': dict(record[0])
-        #          }, record[1], record[2]) for record in cursor]
         result = []
         for record in cursor:
             node: Node = record[0]
@@ -241,152 +310,9 @@ class SoftwareKGSearcher:
             ))
         return result
 
-    def find_all_concept_from_sentence(self, sentence):
-        """
-        给一段文本，返回在KG里面出现的概念
-        """
-        concepts = []
-        sentence = sentence.lower().translate(str.maketrans('', '', string.punctuation)).strip("\n")
-        doc = self.nlp(sentence)
-        words = []
-        for token in doc:
-            words.append(token.lemma_)
-        # words = sentence.strip("\n").lower().split()
-        for i, word in enumerate(words):
-            concept = word
-            if i == len(words) - 1:
-                break
-            for j in range(i + 1, i + 5):
-                if j < len(words):
-                    concept += " " + words[j]
-                if self.is_exist_concept_local(concept):
-                    concepts.append(concept)
-        for word in words:
-            if not self.is_concept_local(word):
-                continue
-            concepts.append(word)
-        return set(concepts)
-
     def is_exit_concept_in_sentence(self, concept, sentence):
         return True if concept in self.find_all_concept_from_sentence(sentence) else False
 
-    def find_longest_valid_concept_from_sentence(self, sentence):
-        """
-        给一段文本，返回在KG里面出现的有效最长概念
-        """
-        concepts = []
-        sentence = sentence.lower().translate(str.maketrans('', '', string.punctuation)).strip("\n")
-        doc = self.nlp(sentence)
-        words = []
-        for token in doc:
-            words.append(token.lemma_)
-        # words = sentence.strip("\n").lower().split()
-        index = 0
-        while index < len(words):
-            concept = words[index]
-            longest_concept = concept
-            for j in range(index + 1, index + 6):
-                if j < len(words):
-                    concept += " " + words[j]
-                    if self.is_concept_local(concept):
-                        longest_concept = concept
-                        index = j
-                    else:
-                        break
-            if self.is_concept_local(longest_concept):
-                concepts.append(longest_concept)
-            index = index + 1
-        return concepts
-
-    def is_stop_word(self, word):
-        return True if word in self.stopwords else False
-
-    def t_find_longest_valid_concept_from_sentence(self, sentence):
-        """
-        给一段文本，返回在KG里面出现的有效最长概念
-        """
-        concepts = []
-        sentence = sentence.lower().translate(str.maketrans('', '', string.punctuation)).strip("\n")
-        doc = self.nlp(sentence)
-        words = []
-        for token in doc:
-            words.append(token.lemma_)
-        # words = sentence.strip("\n").lower().split()
-        index = 0
-        words = ['data' if i == 'datum' else i for i in words]
-        while index < len(words):
-            concept = words[index]
-            longest_concept = concept
-            for j in range(index + 1, index + 6):
-                if j < len(words):
-                    concept += " " + words[j]
-                    if self.is_concept(concept):
-                        longest_concept = concept
-                        index = j
-                    else:
-                        break
-            if self.is_concept(longest_concept):
-                concepts.append(longest_concept)
-            index = index + 1
-        return concepts
-
-    def find_longest_valid_concept_non_stop_words_from_sentence(self, sentence):
-        """
-        给一段文本，返回在KG里面出现的有效最长概念
-        """
-        concepts = []
-        sentence = sentence.lower().translate(str.maketrans('', '', string.punctuation)).strip("\n")
-        doc = self.nlp(sentence)
-        words = []
-        for token in doc:
-            words.append(token.lemma_)
-        index = 0
-        while index < len(words):
-            concept = words[index]
-            longest_concept = concept
-            for j in range(index + 1, index + 6):
-                if j < len(words):
-                    concept += ' ' + words[j]
-                    if self.is_concept_local(concept):
-                        longest_concept = concept
-                        index = j
-                    else:
-                        break
-            if self.is_concept_local(longest_concept) and longest_concept not in self.stopwords:
-                concepts.append(longest_concept)
-            index = index + 1
-        return concepts
-
-    def find_longest_concept_from_sentence(self, sentence):
-        """
-        给一段文本，返回在KG里面出现的最长概念
-        """
-        concepts = []
-        sentence = sentence.lower().translate(str.maketrans('', '', string.punctuation)).strip("\n")
-        doc = self.nlp(sentence)
-        words = []
-        for token in doc:
-            words.append(token.lemma_)
-        # words = sentence.strip("\n").lower().split()
-        index = 0
-        while index < len(words):
-            concept = words[index]
-            longest_concept = concept
-            for j in range(index + 1, index + 6):
-                if j < len(words):
-                    concept += " " + words[j]
-                    if self.is_exist_concept_local(concept):
-                        longest_concept = concept
-                        index = j
-                    else:
-                        break
-            if self.is_concept_local(longest_concept):
-                concepts.append((longest_concept, 'valid'))
-            else:
-                concepts.append((longest_concept, 'invalid'))
-            index = index + 1
-        return concepts
-
     def get_random_node(self):
         query = 'MATCH (n:entity) RETURN n.id as id, labels(n) as labels, properties(n) as properties LIMIT 50'
         cursor: Cursor = self.graph.run(query)
@@ -502,3 +428,143 @@ class SoftwareKGSearcher:
             if upper_concept in node2_upper_concepts:
                 upper_concepts.append(upper_concept)
         return upper_concepts
+
+    # TODO: relations func update next time
+
+    def get_all_in_relations(self, node_id):
+        return None
+        # if node_id not in self.graph.nodes:
+        #     return set()
+        # return {(r[0], r[2], r[1]) for r in self.graph.in_edges(node_id, keys=True)}
+
+    def get_in_relations(self, concept):
+        return None
+        # node_id = self.get_id_by_concept(concept)
+        # relations = self.graph.get_all_in_relations(node_id)
+        # results = []
+        # for relation in relations:
+        #     tup = self.process_relation(relation)
+        #     results.append(tup)
+        # return results
+
+    def find_in_is_a_relation_concept(self, start_word):
+        return None
+        # in_relations = self.get_in_relations(start_word)
+        # result = []
+        # for relation in in_relations:
+        #     if relation[1] == 'is a':
+        #         result.append(relation[0])
+        # return result
+
+    def get_all_out_relations(self, node_id):
+        return None
+        # if node_id not in self.graph.nodes:
+        #     return set()
+        # return {(r[0], r[2], r[1]) for r in self.graph.out_edges(node_id, keys=True)}
+
+    def get_out_relations(self, concept):
+        return None
+        # id = self.get_id_by_concept(concept)
+        # relations = self.graph.get_all_out_relations(id)
+        # results = []
+        # for relation in relations:
+        #     tup = self.process_relation(relation)
+        #     results.append(tup)
+        # return results
+
+    def find_out_is_a_relation_concept(self, start_word):
+        return None
+        # out_relations = self.get_out_relations(start_word)
+        # result = []
+        # for relation in out_relations:
+        #     if relation[1] == 'is a':
+        #         result.append(relation[2])
+        # return result
+
+    def find_out_facet_of_relation_concept(self, start_word):
+        return None
+        # out_relations = self.get_out_relations(start_word)
+        # result = []
+        # for relation in out_relations:
+        #     if relation[1] == 'facet of':
+        #         result.append(relation[2])
+        # return result
+
+    def find_out_facet_of_relation_concept_by_id(self, id):
+        return None
+        # start_word = self.get_concept_by_id(id)
+        # out_relations = self.get_out_relations(start_word)
+        # result = []
+        # for relation in out_relations:
+        #     if relation[1] == 'facet of':
+        #         node = self.get_node_by_concept(relation[2])
+        #         result.append(node)
+        # return result
+
+    def find_out_is_a_relation_concept_by_id(self, id):
+        return None
+        # start_word = self.get_concept_by_id(id)
+        # # print(start_word)
+        # out_relations = self.get_out_relations(start_word)
+        # result = []
+        # for relation in out_relations:
+        #     if relation[1] == 'is a':
+        #         node = self.get_node_by_concept(relation[2])
+        #         result.append(node)
+        # return result
+
+
+class SoftwareKGSearcherLocal(SoftwareKGSearcher):
+    def __init__(self):
+        super(SoftwareKGSearcherLocal, self).__init__()
+        self.concepts = self.__construct_concepts()
+        self.trie: Buildtrim = Buildtrim.load(str(Path(DATA_DIR) / 'sckg.trie'))
+
+    @staticmethod
+    def __construct_concepts():
+        if (Path(DATA_DIR) / 'concept.bin').exists():
+            with open(Path(DATA_DIR) / 'concept.bin', 'rb') as rf:
+                return pickle.load(rf)
+        concepts = {}
+        with open(Path(DATA_DIR) / 'concept.csv', 'r', encoding='utf-8') as rf:
+            reader = csv.reader(rf)
+            next(reader)
+            for row in reader:
+                concepts[row[2]] = (row[1], row[0])
+        with open(Path(DATA_DIR) / 'concept.bin', 'wb') as wf:
+            pickle.dump(concepts, wf)
+        return concepts
+
+    def is_exist_concept(self, concept):
+        return self.concepts.__contains__(concept)
+
+    def get_id_by_concept(self, concept):
+        if not self.is_exist_concept(concept):
+            return None
+        return self.concepts[concept][0]
+
+    def get_concept_labels(self, concept):
+        if not self.is_exist_concept(concept):
+            return None
+        return self.concepts[concept][1]
+
+    def is_action(self, word):
+        labels = self.get_concept_labels(word)
+        if not labels:
+            return False
+        return True if 'action' in labels else False
+
+    def is_characteristic(self, word):
+        labels = self.get_concept_labels(word)
+        if not labels:
+            return False
+        return True if 'characteristic' in labels else False
+
+    def is_concept(self, word):
+        labels = self.get_concept_labels(word)
+        if not labels:
+            return False
+        return False if 'action' in labels or 'characteristic' in labels else True
+
+    def find_include_prefix_concept(self, concept):
+        return self.trie.tree.items(prefix=concept)
